from bdaserviceutils import get_args, get_cmd_arg
from pysparkutilities.spark_initializer import spark_initializer
from pysparkutilities import ds_initializer
from .generic_service import GenericService
from alidaargparser import get_asset_property


class SparkService(GenericService):
    def __init__(self, parser, additional_config=[]):
        super().__init__(parser)

        self._args = get_args()
        print(self._args)

        #additional_config = additional_config.append(('spark.jars.packages', 'io.prestosql:presto-jdbc:350'))
        self.spark = spark_initializer("Test-model", self._args, additional_config=additional_config)

    def get_dataset(self, name="input_dataset"):
        data = ds_initializer.load_dataset(sc=self.spark, name=name, read_all=False)
        return data

    def save_dataset(self, dataset, name="output_dataset"):
        ds_initializer.save_dataset(sc=self.spark,name=name, df=dataset)#, output_dest=self._args['output-dataset'])

    def download_model(self, name='input_model'):
        # If hdfs is specified use it, else running locally
        if get_cmd_arg('hdfsUrl') is None:
            hdfs_url = ""
        else:
            hdfs_url = get_cmd_arg('hdfsUrl')

        return hdfs_url + get_asset_property(asset_name=name)

    def upload_model(self, model, name='output_model'):
        # If hdfs is specified use it, else running locally
        if get_cmd_arg('hdfsUrl') is None:
            hdfs_url = ""
        else:
            hdfs_url = get_cmd_arg('hdfsUrl')

        model_path = hdfs_url + get_asset_property(asset_name=name)
        model.write().overwrite().save(model_path)



from bdaserviceutils import get_args


class GenericService():
    def __init__(self, parser):
        super().__init__()
        
        # Raw args
        self._args = get_args()
        
        self.parser = parser
        # Parsed args
        self.args = self.parser.parse_args()


    def get_datasets(self):
        datasets = {}
        for dataset in self.parser.input_datasets:
            datasets[dataset.name] = self.get_dataset(dataset.name)

        # If only one dataset, return it directly
        if len(datasets) == 1:
            return datasets[next(iter(datasets))]

        return datasets

    def save_datasets(self, datasets):
        if isinstance(datasets, dict):
            for dataset in self.parser.output_datasets:
                self.save_dataset(datasets[dataset.name], dataset.name)
        # If only one dataset a dictionary is not expected
        else:
            self.save_dataset(datasets, self.parser.output_datasets[0].name)


    def download_models(self):
        models = {}
        for model in self.parser.input_models:
            models[model.name] = self.download_model(model.name)
        
        # If only one model, return it directly
        if len(models) == 1:
            return models[next(iter(models))]
        
        return models

    def upload_models(self, models):     
        if isinstance(models, dict):
            for model in self.parser.output_models:
                self.upload_model(models[model.name], model.name)   
        # If only one dataset a dictionary is not expected
        else:
            self.upload_model(models, self.parser.output_models[0].name)



import os

class IO:
    @staticmethod
    def dataset_to_dataset(func):
        
        def wrapper(*args):
            self = args[0]
            datasets = self.get_datasets()
            resulting_dataset = func(self, datasets)
            self.save_dataset(resulting_dataset)

        return wrapper


    @staticmethod
    def dataset_to_model(func):
        
        def wrapper(*args):
            self = args[0]
            datasets = self.get_datasets()
            model_path = func(self, datasets)
            self.upload_model(model_path)

        return wrapper


    @staticmethod
    def dataset_and_model_to_dataset(func):
        
        def wrapper(*args):
            self = args[0]
            dataset = self.get_dataset()
            model_path = self.download_model()
            resulting_dataset = func(self, dataset, model_path)
            self.save_dataset(resulting_dataset)

        return wrapper

    @staticmethod
    def dataset_to_dataset_and_model(func):
        
        def wrapper(*args):
            self = args[0]
            dataset = self.get_dataset()
            resulting_dataset, resulting_model = func(self, dataset)
            self.save_dataset(resulting_dataset)
            self.upload_model(resulting_model)

        return wrapper

    @staticmethod
    def dataset_and_model_to_dataset_and_model(func):
        
        def wrapper(*args):
            self = args[0]
            dataset = self.get_dataset()
            model_path = self.download_model()
            resulting_dataset, resulting_model = func(self, dataset, model_path)
            self.save_dataset(resulting_dataset)
            self.upload_model(resulting_model)

        return wrapper

    @staticmethod
    def io(func):
        def wrapper(*args):
            self = args[0]
            
            args_to_pass = {}

            if len(self.parser.input_datasets) > 0:
                datasets = self.get_datasets()
                args_to_pass['datasets' if len(self.parser.input_datasets) > 1 else 'dataset'] = datasets
            
            if len(self.parser.input_models) > 0:
                models = self.download_models()
                args_to_pass['models' if len(self.parser.input_models) > 1 else 'model'] = models

            result = func(self, **args_to_pass)

            if len(self.parser.output_datasets) > 0:
                if isinstance(result, tuple):
                    datasets = result[0]
                else:
                    datasets = result
                self.save_datasets(datasets)
            
            if len(self.parser.output_models) > 0:
                if isinstance(result, tuple):
                    models = result[1]
                else:
                    models = result            
                self.upload_models(models)

        return wrapper




from .pandas_service import PandasService
from .spark_service import SparkService
from .io import IO
from .processor_service import ProcessorService
from .source_service import SourceService
from .sink_service import SinkService



from .generic_service import GenericService
from bdaserviceutils import HttpHealthServer

class StreamingService(GenericService):

    def __init__(self, parser):
        super().__init__(parser)
        HttpHealthServer.run_thread()



from abc import abstractclassmethod
from .streaming_service import StreamingService
from bdaserviceutils import get_kafka_binder_brokers, get_output_channel
from kafka import KafkaProducer
from abc import ABC, abstractclassmethod
import json


class SourceService(ABC, StreamingService):
    
    alida_service_mode = "source"
    
    def __init__(self, parser):
        super().__init__(parser)
        self.producer = KafkaProducer(bootstrap_servers=[get_kafka_binder_brokers()])

    
    @abstractclassmethod
    def run(self):
        pass

    def send_message(self, message):
        self.producer.send(get_output_channel(), json.dumps(message).encode('utf-8'))
